# File: ATWorkgroup.py
#
# Copyright (c) 2006 by [mxm.dk]
# Generator: ArchGenXML Version 1.4.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.


__author__ = """Max M <maxm@mxm.dk>"""
__docformat__ = 'plaintext'

# zope
from AccessControl import ClassSecurityInfo

# CMF
from Products.CMFCore.utils import getToolByName, UniqueObject

# Plone
from Products.CMFPlone.migrations.migration_util import safeEditProperty

# Archetypes
from Products.Archetypes.atapi import *

# atct
from Products.ATContentTypes.content.folder import ATBTreeFolder

# product
from Products.ATWorkgroup.config import *
from Products.ATWorkgroup.EmailMixin import EmailMixin

# portlet utilities
try:
    from plone.app.portlets import utils as portletutils
except:
    portletutils = False

class ATWorkgroup(ATBTreeFolder, EmailMixin):
    
    security = ClassSecurityInfo()
    
    __implements__ = (getattr(ATBTreeFolder,'__implements__',()),)

    # This name appears in the 'add' box
    archetype_name = 'Workgroup'

    meta_type = 'ATWorkgroup'
    portal_type = 'ATWorkgroup'
    global_allow = 1
    content_icon = 'Workgroup.gif'
    typeDescription = "ATWorkgroup"
    typeDescMsgId = 'description_edit_atworkgroup'
    _atct_newTypeFor = {'portal_type':'Workgroup', 'meta_type':'Workgroup'}
    
    actions = (
        {
            'name'         : 'Add Members',
            'id'           : 'Workgroupmembers',
            'action'       : 'string:$object_url/WorkgroupMembers_Form',
            'permissions'  : (Workgroup_membershipPermission,),
            'category'     : 'object',
        },
        {
            'name'         : 'Workgroup Roles',
            'id'           : 'workgroupLocalRoles',
            'action'       : 'string:$object_url/Workgroup_local_roles',
            'permissions'  : (Workgroup_membershipPermission,),
            'category'     : 'object',
        },
        {
            'name'         : 'Mailform',
            'id'           : 'mailform',
            'action'       : 'string:$object_url/Workgroup_mailForm',
            'permissions'  : (Workgroup_mailPermission,),
            'category'     : 'object',
            'visible'      : 0,
        },
        {
            'name'         : 'Mailarchive',
            'id'           : 'mailarchive',
            'action'       : 'string:$object_url/Workgroup_mailArchive',
            'permissions'  : (Workgroup_editPermission,),
            'category'     : 'object',
        },
    )

    security.declarePublic('getWorkgroup')
    def getWorkgroup(self):
        "returns this Workgroup"
        return self

    security.declarePrivate('getWorkgrouptool')
    def getWorkgrouptool(self):
        return getToolByName(self, 'portal_workgrouptool')

    security.declarePrivate('at_post_create_script')
    def at_post_create_script(self):
        "Creation script"
        # Add workgroup protlets
        groups_portlets = (
            'here/portlet_Workgroup_members/macros/portlet',
            'here/portlet_Workgroup_mail/macros/portlet',
        )
        safeEditProperty(self, 'right_slots', groups_portlets, 'lines')
        # Possibly add creator to workgroup
        wtool = self.getWorkgrouptool()
        if wtool.getAddCreatorToGroup():
            portal_membership = getToolByName(self, 'portal_membership')
            member = portal_membership.getAuthenticatedMember()
            self.addGroupmembers(member)
        # convert old style portlets to new
        # yeas this should probably be made into new style portlets
        # but that is to much work to bother :-s
        if portletutils:
            portletutils.convert_legacy_portlets(self)



    #####################################
    # Membership and local role setting methods
    
    # Basically they shadows the methods in the tool.
    # but we need security checks at this level because of local roles etc.
    # So this is easier for all. Well for me at least
    
    security.declareProtected(Workgroup_membershipPermission, 'delGroupmembers')
    def addGroupmembers(self, members):
        "Adds members to a group"
        self.getWorkgrouptool().addGroupmembers([self], members)

    security.declareProtected(Workgroup_membershipPermission, 'delGroupmembers')
    def delGroupmembers(self, members):
        "Delete members from a group"
        for member in members:
            self.manage_delLocalRoles([member])
        self.getWorkgrouptool().delGroupmembers([self], members)

    security.declareProtected(Workgroup_membershipPermission, 'getGroupMembers')
    def getGroupMembers(self):
        "Returns all the members of the workgroup"
        return self.getWorkgrouptool().getGroupMembers(self)

    security.declarePublic('getAccesibleGroupMembers')
    def getAccesibleGroupMembers(self):
        "Returns the accessible members of the workgroup"
        return self.getWorkgrouptool().getAccesibleGroupMembers(self)



    security.declarePublic('getHomeFolders')
    def getHomeFolders(self):
        "Returns the homefolder of the members of the workgroup"
        # perhaps this should be a skin method?
        # at least members of the groups should get the full list?
        portal_membership = getToolByName(self, 'portal_membership')
        members = self.getWorkgroupMembers()
        result = []
        for member in members:
            member_id = member.getMemberId()
            homefolder = portal_membership.getHomeFolder(member_id)
            if homefolder is not None:
                result.append(homefolder)
        return result

    security.declareProtected(Workgroup_membershipPermission, 'updateLocalroles')
    def updateLocalroles(self, member_id, old_local_roles, new_local_roles):
        "Updates the local roles for a member"
        # first I filter the roles, so that only the roles that the 
        # member has in this context can be used. Just in case of
        # a malicious member. For normal use it has no value.
        mtool = getToolByName(self, 'portal_membership')
        candidates = mtool.getCandidateLocalRoles(self)
        old_local_roles = [role for role in old_local_roles if role in candidates]
        new_local_roles = [role for role in new_local_roles if role in candidates]
        # *_local_roles are only partial lists because member might not 
        # have right to change some roles, so these lists should be 
        # compared to "local_roles_for_userid". So I need to hold the 
        # tongue-in-cheek here. Please don't disturb while I write this.
        roles_to_delete = [role for role in old_local_roles if not role in new_local_roles]
        roles_to_add = [role for role in new_local_roles if not role in old_local_roles]
        # get complete list of local roles from the object
        local_roles_for_userid = self.get_local_roles_for_userid(member_id)
        # delete & add roles from form to make new complete rolelist
        new_local_roles = [role for role in local_roles_for_userid if not role in roles_to_delete]
        new_local_roles += [role for role in roles_to_add if not role in new_local_roles]
        # we cannot set local roles when there aren't any.
        # Then we just delete all the users local roles
        if not new_local_roles:
            self.manage_delLocalRoles([member_id])
        else:
            self.manage_setLocalRoles(member_id, new_local_roles)
        # we must adjust visibility according to the new settings
        self.reindexObjectSecurity(skip_self=True)
        # Ok you can talk again now


    ################################################################################
    # Workgroup compatible methods. Made only for backwards compatibility. They Will 
    # be removed at some time! DO NOT USE THEM ANYMORE...!!!
    
    security.declareProtected(Workgroup_membershipPermission, 'addGroupmembersById')
    addGroupmembersById = addGroupmembers

    security.declareProtected(Workgroup_membershipPermission, 'delGroupmembersById')
    delGroupmembersById = delGroupmembers

    security.declarePrivate('getWorkgroupMembers')
    getWorkgroupMembers = getGroupMembers

    security.declarePublic('getAccesibleWorkgroupMembers')
    getAccesibleWorkgroupMembers = getAccesibleGroupMembers
    ################################################################################




registerType(ATWorkgroup,PROJECTNAME)



