from cStringIO import StringIO
import logging
LOG = logging.getLogger('ATCT.migration')

from Products.CMFCore.utils import getToolByName

from Products.ATContentTypes.migration.common import migratePortalType
from Products.ATContentTypes.migration.common import registerATCTMigrator
from Products.ATContentTypes.migration.walker import CatalogWalkerWithLevel
from Products.ATContentTypes.migration.atctmigrator import FolderMigrator

from Products.ATWorkgroup.ATWorkgroup import ATWorkgroup

class WorkgroupMigrator(FolderMigrator):

    
    def custom(self):
        "Custom migration stuff"
        members = self.old.getWorkgroupMembers()
        self.new.addGroupmembers(members)
        if hasattr(self.old, '_messageStorage'):
            self.new._messageStorage = self.old._messageStorage
        if hasattr(self.old, '_msgCounter'):
            self.new._msgCounter = self.old._msgCounter
        
        
    def last_migrate_RenamePortlets(self):
        "Renames portlets"
        old_members = 'here/Workgroup_portlet_members/macros/portlet'
        new_members = 'here/portlet_Workgroup_members/macros/portlet'
        old_mail = 'here/Workgroup_portlet_mail/macros/portlet'
        new_mail = 'here/portlet_Workgroup_mail/macros/portlet'
        right_slots = list(getattr(self.new, 'right_slots', []))
        changed = 0
        for old, new in ((old_members,new_members),(old_mail,new_mail)):
            if old in right_slots:
                changed = 1
                i = right_slots.index(old)
                right_slots[i] = new
        if changed:
            self.new.right_slots = tuple(right_slots)

##    # you can add your own migrations steps here
##    def last_migrate_YourOwnChanges(self):
##        "Do custom stuff"
##        self.new.property = self.old.property


registerATCTMigrator(WorkgroupMigrator, ATWorkgroup)


def migrateAll(portal, **kwargs):
    LOG.debug('Starting ATWorkgroup migration')
    kwargs = kwargs.copy()
    
    atct = getToolByName(portal, 'portal_atct')
    kwargs['use_catalog_patch'] = bool(atct.migration_catalog_patch)
    kwargs['transaction_size'] = int(atct.migration_transaction_size)
    transaction = atct.migration_transaction_style
    if transaction == "full transaction":
        kwargs['full_transaction'] = True
        kwargs['use_savepoint'] = False
    elif transaction == "savepoint":
        kwargs['full_transaction'] = False
        kwargs['use_savepoint'] = True
    else:
        kwargs['full_transaction'] = False
        kwargs['use_savepoint'] = False
    
    for remove in ('src_portal_type', 'dst_portal_type'):
        if remove in kwargs:
            del kwarg[remove]


    out = StringIO()
    migrator = WorkgroupMigrator
    src_portal_type = migrator.src_portal_type
    dst_portal_type = migrator.dst_portal_type
    ttool = getToolByName(portal, 'portal_types')
    if (ttool.getTypeInfo(src_portal_type) is None or
       ttool.getTypeInfo(dst_portal_type) is None):
        
        LOG.debug('Missing FTI for %s or %s'%(src_portal_type, dst_portal_type))
        print >>out, ("Couldn't migrate src_portal_type due to missing FTI")
    else:
        migratePortalType(portal, src_portal_type, dst_portal_type, out=out,
                          migrator=migrator, **kwargs)

    # Migrating tool and portal settings

    print >>out, 'Migrating tool settings.'
    old = portal.mxm_workgroups
    new = portal.portal_workgrouptool
    new.setInitial_LocalRoles(old.initial_local_roles)
    new.setAddCreatorToGroup(old.add_creator_as_groupmember)


    print >>out, 'Updating the catalog.'
    portal.portal_catalog.refreshCatalog(clear=1)

    LOG.debug('Finished ATWorkgroup type migration')
    
    return out.getvalue()
