from Products.CMFCore.utils import getToolByName

def copyWorkflow(wf_tool, old_id, new_id):
    "Copy an existing workflow to a new id. Returns the new copy"
    objects = wf_tool.manage_copyObjects([old_id])
    wf_tool.manage_pasteObjects(objects)
    wf_tool.manage_renameObject('copy_of_%s' % old_id, new_id)
    return getattr(wf_tool, new_id)


def setupWorkflows(self, out):
    "Set up workflows for the workgroups"
    
    # get the workflow tool
    wftool = getToolByName(self, 'portal_workflow')
    
    # set up some shared values
    group_state = 'visible4group'
    group_transition = 'publish2group'
    
    
    ###################################################################
    # add the workgroup content workflow based on plone folder workflow
    
    # create new workflow
    workgroup_workflow_id = 'workgroup_workflow'
    workgroup_workflow_exists = workgroup_workflow_id in wftool.getWorkflowIds()
    if workgroup_workflow_exists:
        out.write("*** %s allready exists. No changes made\n" % workgroup_workflow_id)
    else:
        workgroup_workflow = copyWorkflow(wftool, 'folder_workflow', workgroup_workflow_id)
        workgroup_workflow.title='Workgroup Workflow [Plone]'
        
        # add transition and state
        workgroup_workflow.states.addState(group_state)
        workgroup_workflow.transitions.addTransition(group_transition)
        
        # set up transition
        publish2group = workgroup_workflow.transitions.publish2group
        TRIGGER_USER_ACTION = 1 # Initiated by user action
        publish2group.setProperties(
            title='Publish to group',
            new_state_id=group_state,
            trigger_type=TRIGGER_USER_ACTION, 
            actbox_name='Visible for group', 
            actbox_url='%(content_url)s/content_publish_form',
            actbox_category='workflow',
            props={
                'guard_permissions':'',
                'guard_roles':'Owner',
                'guard_expr':"python:getattr(here, 'getWorkgroup')", # only relevant under a workgroup
            }, 
            description='Publish content to group only'
        )

        # set up states
        visible4group = workgroup_workflow.states.visible4group
        visible4group.setProperties(transitions=['hide','publish','show'])
        # add transition
        workflow_permissions = [
            ['Access contents information',0,['GroupMember', 'Manager', 'Owner']],
            ['List folder contents', 0,  ['GroupMember', 'Manager', 'Owner']],
            ['Modify portal content', 0,  ['Manager', 'Owner']],
            ['View', 0,  ['GroupMember', 'Manager', 'Owner']],
        ]
        for p, a, r in workflow_permissions:
            visible4group.setPermission(permission=p, acquired=a, roles=r)
    
        # set up properties
        private = workgroup_workflow.states.private
        private.setProperties(
            title=private.title, 
            transitions=['publish',group_transition,'show'], 
            description=private.description
        )
        published = workgroup_workflow.states.published
        published.setProperties(
            title=published.title, 
            transitions=['hide',group_transition,'retract'], 
            description=published.description
        )
        visible = workgroup_workflow.states.visible
        visible.setProperties(
            title=visible.title, 
            transitions=['hide','publish',group_transition], 
            description=visible.description
        )
        out.write( "Added Workgroup workflow.\n" )
        
        # set default workflow for workgroup
        wftool.setChainForPortalTypes(['ATWorkgroup'], workgroup_workflow_id)
        out.write( "Made Workgroup workflow default for workgroup\n" )



    ###################################################################
    # add the workgroup content workflow based on the default plone workflow
    
    workgroup_content_workflow_id = 'workgroup_content_workflow'
    workgroup_content_workflow_exist = workgroup_content_workflow_id in wftool.getWorkflowIds()
    if workgroup_content_workflow_exist:
        out.write("*** %s allready exists. No changes made\n" % workgroup_content_workflow_id)
    else:
        workgroup_content_workflow = copyWorkflow(wftool, 'plone_workflow', workgroup_content_workflow_id)
        workgroup_content_workflow.title='Workggroup Content Workflow [Plone]'
        
        # add transition and state
        workgroup_content_workflow.states.addState(group_state)
        workgroup_content_workflow.transitions.addTransition(group_transition)
    
        # set up transition
        publish2group = workgroup_content_workflow.transitions.publish2group
        TRIGGER_USER_ACTION = 1 # Initiated by user action
        publish2group.setProperties(
            title='Publish to group',
            new_state_id=group_state,
            trigger_type=TRIGGER_USER_ACTION, 
            actbox_name='Visible for group', 
            actbox_url='%(content_url)s/content_publish_form',
            actbox_category='workflow',
            props={
                'guard_permissions':'',
                'guard_roles':'Owner',
                'guard_expr':'here/getWorkgroup | nothing',
            }, 
            description='Publish content to group only'
        )
    
        # set up states
        visible4group = workgroup_content_workflow.states.visible4group
        visible4group.setProperties(transitions=['hide','publish','show','submit'])
        # add transition
        workflow_permissions = [
            ['Access contents information',0, ['GroupMember', 'Manager', 'Owner']],
            ['Change portal events', 0,  ['Manager', 'Owner']],
            ['Modify portal content', 0,  ['Manager', 'Owner']],
            ['View', 0,  ['GroupMember', 'Manager', 'Owner']],
        ]
        for p, a, r in workflow_permissions:
            visible4group.setPermission(permission=p, acquired=a, roles=r)
    
        # set up properties
        private = workgroup_content_workflow.states.private
        private.setProperties(
            title=private.title, 
            transitions=[group_transition,'show'], 
            description=private.description
        )
        published = workgroup_content_workflow.states.published
        published.setProperties(
            title=published.title, 
            transitions=['reject','retract'], 
            description=published.description
        )
        visible = workgroup_content_workflow.states.visible
        visible.setProperties(
            title=visible.title, 
            transitions=['hide','publish',group_transition,'submit'], 
            description=visible.description
        )
        out.write("Added Workgroup content workflow.\n")

        # set default workflow for workgroup content / overwrite the default workflow
        wftool.setDefaultChain(workgroup_content_workflow_id)
        out.write( "Made Workgroup content workflow default for content\n" )
    
    # Update security settings if any workflow has been added.
    if not workgroup_workflow_exists or not workgroup_content_workflow_exist:
        update_count = wftool.updateRoleMappings()
        out.write("Updated the security settings on %s objects\n" % update_count)
