﻿# -*- coding: utf-8 -*-

# File: workgrouptool.py
#
# Copyright (c) 2006 by [mxm.dk]
# Generator: ArchGenXML Version 1.4.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Max M <maxm@mxm.dk>"""
__docformat__ = 'plaintext'

# python
import struct
from cStringIO import StringIO
from types import StringTypes, ListType, TupleType
#SequenceTypes = (ListType, TupleType)

# Zope
from BTrees.OOBTree import OOBTree
from AccessControl import ClassSecurityInfo
View = 'View' # yeah thats not correct

# CMF
from Products.CMFCore.utils import getToolByName, UniqueObject
from Products.CMFCore.ActionProviderBase import ActionProviderBase
try: # new cmf
    from Products.CMFCore.ActionInformation import ActionInformation
except:
    from Products.CMFCore.ActionsTool import ActionInformation
from Products.CMFCore.Expression import Expression

# Archetypes
from Products.Archetypes.atapi import *
from Products.Archetypes.config import UID_CATALOG

# Product
from Products.ATWorkgroup.config import *
from WorkgroupRelations import WorkgroupRelations


schema = Schema((

    StringField(
        name='title',
        widget=StringWidget(
            label='Title',
            label_msgid='ATWorkgroup_label_title',
            i18n_domain='ATWorkgroup',
            visible={'edit':'hidden', 'view':'invisible'},
        ),
        accessor='Title',
        default='Workgroup Tool',
    ),

    LinesField(
        name='initial_LocalRoles',
        widget=MultiSelectionWidget(
            label='Initial Local Roles',
            label_msgid='ATWorkgroup_label_initialLocalRoles',
            description='The roles a new member will get in a workgroup.',
            description_msgid='ATWorkgroup_help_initialLocalRoles',
            i18n_domain='ATWorkgroup',
        ),
        default=['GroupMember'],
        vocabulary='allRolesDisplayList',
    ),

    BooleanField(
        name='addCreatorToGroup',
        widget=BooleanWidget(
            label='Add creator as groupmember',
            label_msgid='ATWorkgroup_label_add_creator_as_groupmember',
            description='Should the creator of the group automatically be added as a member.',
            description_msgid='ATWorkgroup_help_add_creator_as_groupmember',
            i18n_domain='ATWorkgroup',
        ),
    ),

),
)




workgrouptool_schema = BaseSchema.copy() + schema.copy()


class workgrouptool(UniqueObject, BaseContent, ActionProviderBase, WorkgroupRelations):
    
    security = ClassSecurityInfo()
    
    __implements__ = (getattr(UniqueObject,'__implements__',()),) + \
                     (getattr(ActionProviderBase,'__implements__',()),) + \
                     (getattr(BaseContent,'__implements__',()),)

    # This name appears in the 'add' box
    archetype_name = 'workgrouptool'

    meta_type = 'workgrouptool'
    portal_type = 'workgrouptool'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 0
    allow_discussion = False
    #content_icon = 'workgrouptool.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "workgrouptool"
    typeDescMsgId = 'description_edit_workgrouptool'
    #toolicon = 'workgrouptool.gif'

    schema = workgrouptool_schema

    actions = (
        {
            'title'        : 'Groups',
            'id'           : 'Workgroup_workgroups',
            'action'       : 'string:$object_url/Workgroup_workgroups',
            'permissions'  : ('Manage Portal',),
            'category'     : 'object',
            'condition'    : '',
            'visible'      : 1,
        },
        {
            'title'        : 'Members',
            'id'           : 'Workgroup_members',
            'action'       : 'string:$object_url/Workgroup_members',
            'permissions'  : ('Manage Portal',),
            'category'     : 'object',
            'condition'    : '',
            'visible'      : 1,
        },
        {
            'title'        : 'My Workgroups',
            'id'           : 'myworkgroups',
            'action'       : 'python:portal.portal_membership.getHomeUrl() + "/Workgroup_myGroups"',
            'permissions'  : (View,),
            'category'     : 'user',
            'condition'    : 'python: member and portal.portal_membership.getHomeFolder()',
            'visible'      : 1,
        },
    )

    manage_options = BaseContent.manage_options + \
                     ActionProviderBase.manage_options

    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseContent.__init__(self,'portal_workgrouptool')
        WorkgroupRelations.__init__(self)


    ##########################
    # private methods
        
    security.declarePublic('allRolesDisplayList')
    def allRolesDisplayList(self):
        "Returns a displaylist with all roles."
        return DisplayList([(r,r) for r in self.get_all_roles()])

    # tool shortcuts

    def mtool(self):
        return getToolByName(self, 'portal_membership')


    def get_all_roles(self):
        "Returns alle roles"
        return self.mtool().getPortalRoles()

    # private helpers

    def _workgroups2uids(self, workgroups):
        """
        Returns list of uid's from workgroups or uids.
        members must have one of these forms:
        members == 'uid
        members == ['uid, 'uid2']
        members == <workgroup object at...>
        members == [<workgroup object at...>, <workgroup object at...>]
        """
        if isinstance(workgroups, StringTypes):
            return [workgroups]
        if hasattr(workgroups, 'UID'):
            return [workgroups.UID()]
        if workgroups and isinstance(workgroups[0], StringTypes):
            return workgroups
        return [workgroup.UID() for workgroup in workgroups]

    def _uids2workgroups(self, uids):
        """
        Returns list of objects from uid's or objects
        members must have one of these forms:
        members == 'uid'
        members == ['uid1', 'uid2']
        members == <workgroup object at...>
        members == [<workgroup object at...>, <workgroup object at...>]
        """
        # check if it is a single string
        if isinstance(uids, StringTypes):
            return self._getByUids(uids)
        # or a single object
        if hasattr(uids, 'UID'):
            return [uids]
        # If first in list is uid assume they are all uids
        if uids and isinstance(uids[0], StringTypes):
            return self._getByUids(uids)
        # assume its a list of objects
        return uids


    def _getByUids(self, uuid):
        tool = getToolByName(self, UID_CATALOG)
        brains = tool(UID=uuid)
        result = []
        for brain in brains:
            obj = brain.getObject()
            if obj is not None:
                result.append(obj)
        return result


    def _members2Ids(self, members):
        """
        Returns id's from member objects or ids.
        members must have one of these forms:
        members == <member object at...>
        members == [<member object at...>, <member object at...>]
        members == 'memberID'
        members == ['memberId1', 'memberId2']
        """
        if not members:
            return []
        if isinstance(members, StringTypes):
            return [members]
        if hasattr(members, 'getMemberId'):
            return [members.getMemberId()]
        if members and isinstance(members[0], StringTypes):
            return members
        return [member.getMemberId() for member in members]


    def _ids2members(self, ids):
        "Returns members from list of member Ids"
        portal_membership = self.mtool()
        members = []
        for id in ids:
            member = portal_membership.getMemberById(id)
            if member:
                members.append(member)
        return members

    def _getAuthenticatedMember(self):
        return getToolByName(self, 'portal_membership').getAuthenticatedMember()

    def _filterAccesible(self, workgroups):
        """
        filter away groups that the authenticated member does not have right 
        to see eg. if a group is in another members folder with private state,
        it cannot be accessed.
        """
        member = self._getAuthenticatedMember()
        result = []
        for workgroup in workgroups:
            if member.has_permission(View, workgroup):
                result.append(workgroup)
        return result


    ########################################################
    # Workgroup specific methods
    # These methods are called by the workgroup content type
    
    # groups or uids and members or memberIds are interchangable

    # handling groups members

    security.declareProtected(WorkgroupTool_editPermission, 'addGroupmembers')
    def addGroupmembers(self, groups, groupMembers):
        "Adds members to a group"
        memberIds = self._members2Ids(groupMembers)
        uids = self._workgroups2uids(groups)
        self._addMembers(uids, memberIds)
        self.setInitialLocalRoles(groups, groupMembers)

    security.declareProtected(WorkgroupTool_editPermission, 'delGroupmembers')
    def delGroupmembers(self, groups, groupMembers):
        "Delete members from a group"
        memberIds = self._members2Ids(groupMembers)
        uids = self._workgroups2uids(groups)
        self._unrelateMembers(uids, memberIds)

    security.declarePublic('getGroupMembers')
    def getGroupMembers(self, groups):
        "Returns the members of a group"
        memberIds = self._getMembers(self._workgroups2uids(groups))
        return self._ids2members(memberIds)

    security.declarePublic('getAccesibleGroupMembers')
    def getAccesibleGroupMembers(self, groups):
        """
        Returns the members of a group, with accessible 
        home folders.
        """
        portal_membership = getToolByName(self, 'portal_membership')
        result = []
        for member in self.getGroupMembers(groups):
            memberid = member.getMemberId()
            if portal_membership.getHomeFolder(id=memberid, verifyPermission=1):
                result.append(member)
        return result



    # handling groups for members

    security.declarePublic('getGroupsForMember')
    def getGroupsForMember(self, member):
        "Returns all the groups for a member"
        memberIds = self._members2Ids(member)
        workgroupuids = self._getWorkgroups(memberIds)
        workgroups = self._getByUids(workgroupuids)
        return workgroups

    security.declarePublic('getAccesibleGroupsForMember')
    def getAccesibleGroupsForMember(self, member):
        """
        Returns all the groups for a member, that the authenticated user has 
        access to. This is practical for user interfaces.
        """
        workgroups = self.getGroupsForMember(member)
        return self._filterAccesible(workgroups)



    # managing local roles when creating new groups or adding members

    security.declarePrivate('setInitialLocalRoles')
    def setInitialLocalRoles(self, groups, members):
        "Set the initial local roles"
        init_roles = self.getInitial_LocalRoles()
        if init_roles:
            groups = self._uids2workgroups(groups)
            memberIds = self._members2Ids(members)
            for group in groups:
                for memberId in memberIds:
                    group.manage_setLocalRoles(memberId, list(init_roles))

    security.declareProtected(WorkgroupTool_editPermission, 'delLocalRoles')
    def delLocalRoles(self, groups, members, reindex=1):
        """ Delete local roles for members member_ids """
        groups = self._uids2workgroups(groups)
        memberIds = self._members2Ids(members)
        for group in groups:
            group.manage_delLocalRoles(userids=memberIds)
            if reindex:
                group.reindexObjectSecurity()

    security.declareProtected(WorkgroupTool_editPermission, 'setLocalroles')
    def setLocalroles(self, groups, members, new_local_roles, reindex=1):
        """sets the local roles for a group member. Previous roles are 
        discarded"""
        groups = self._uids2workgroups(groups)
        memberIds = self._members2Ids(members)
        for group in groups:
            group.manage_setLocalRoles(memberIds, list(new_local_roles))
            if reindex:
                group.reindexObjectSecurity()



    ##############
    # View methods

    def _dict_size(self, a_dict):
        "Calculates the aproximate size of a dict of strings."
        sizeof_PyObject = len(struct.pack('P', 0))
        s_data = 0
        for k, v in a_dict.items():
            # Size of strings
            s_data += len(k) + sizeof_PyObject
            s_data += len(v) + sizeof_PyObject
        # size of dictionary overhead
        s_pointers = len(a_dict) * 1.5 * sizeof_PyObject
        return s_data + int(s_pointers)




    security.declareProtected(WorkgroupTool_editPermission, 'workgroups_html')
    def workgroups_html(self):
        "Renders workgroups list as html"
        out = StringIO()
        out.write('<b>id:</b> %s<br>\n' % self.getId())
        out.write('<b>title:</b> %s<br><br>\n' % self.title_or_id())
        out.write('<b>Number of related objects:</b> %s<br>\n' % len(self._workgroups.relations))
        out.write('<b>Size of relations object (approximate) bytes:</b> %s<br><br>\n' % self._dict_size(self._workgroups.relations))
        uids = self._getAllWorkgroups()
        workgroups = self._uids2workgroups(uids)
        for workgroup in workgroups:
            out.write('<b>%s</b><br>\n' % workgroup.title_or_id())
            for member in self.getGroupMembers(workgroup):
                out.write('&nbsp;&nbsp;&nbsp;&nbsp;%s<br>\n' % member.getMemberId())
        return out.getvalue()

    security.declareProtected(WorkgroupTool_editPermission, 'Workgroup_workgroups')
    def Workgroup_workgroups(self):
        """
        Renders the relations in html. The result can get quite big 
        with many relations (several MBytes), so this is the most 
        efficient approach.
        If it should be inserted into a dtml or zpt document, it would 
        at least double the needed memory. I know this for a fact :-(
        """
        title = self.title_or_id()
        charset = self.portal_properties.site_properties.default_charset
        header = """
            <html>
            <head>
                <title>Groups in: %(title)s</title>
                <meta http-equiv="Content-Type" content="text/html;charset=%(charset)s" />
            </head>
            <body>
            <h2>Groups in: %(title)s</h2>
        """ % locals()
        footer = """
            </body>
            </html>
        """
        out = StringIO()
        out.write(header)
        out.write(self.workgroups_html())
        out.write(footer)
        return out.getvalue()




    security.declareProtected(WorkgroupTool_editPermission, 'members_html')
    def members_html(self):
        "Renders workgroups list as html"
        out = StringIO()
        out.write('<b>id:</b> %s<br>\n' % self.getId())
        out.write('<b>title:</b> %s<br><br>\n' % self.title_or_id())
        out.write('<b>Number of related objects:</b> %s<br>\n' % len(self._members.relations))
        out.write('<b>Size of relations object (approximate) bytes:</b> %s<br><br>\n' % self._dict_size(self._members.relations))
        ids = self._getAllMembers()
        members = self._ids2members(ids)
        for member in members:
            out.write('<b>%s</b> (%s)<br>\n' % (member.getProperty('fullname', '-'), member.getMemberId()))
            for workgroup in self.getGroupsForMember(member):
                out.write('&nbsp;&nbsp;&nbsp;&nbsp;%s<br>\n' % workgroup.title_or_id())
        return out.getvalue()



    security.declareProtected(WorkgroupTool_editPermission, 'Workgroup_members')
    def Workgroup_members(self):
        """
        Renders the relations in html. The result can get quite big 
        with many relations (several MBytes), so this is the most 
        efficient approach.
        If it should be inserted into a dtml or zpt document, it would 
        at least double the needed memory. I know this for a fact :-(
        """
        title = self.title_or_id()
        charset = charset = self.portal_properties.site_properties.default_charset
        header = """
            <html>
            <head>
                <title>Members in: %(title)s</title>
                <meta http-equiv="Content-Type" content="text/html;charset=%(charset)s" />
            </head>
            <body>
            <h2>Members in: %(title)s</h2>
        """ % locals()
        footer = """
            </body>
            </html>
        """
        out = StringIO()
        out.write(header)
        out.write(self.members_html())
        out.write(footer)
        return out.getvalue()



    ################################################################################
    # Workgroup compatible methods. Made only for backwards compatibility. They Will 
    # be removed at some time! DO NOT USE THEM ANYMORE...!!!
    
    security.declareProtected(WorkgroupTool_editPermission, 'addGroupmembersById')
    addGroupmembersById = addGroupmembers

    security.declareProtected(WorkgroupTool_editPermission, 'delGroupmembersById')
    delGroupmembersById = delGroupmembers

    security.declarePublic('getGroupsForMemberId')
    getGroupsForMemberId = getGroupsForMember

    security.declarePublic('getAccesibleGroupsForMember')
    getAccesibleGroupsForMemberId = getAccesibleGroupsForMember

    security.declarePublic('getMembersOfGroup')
    getMembersOfGroup = getGroupMembers

    security.declarePublic('getAccesibleMembersOfGroup')
    getAccesibleMembersOfGroup = getAccesibleGroupMembers

    ################################################################################


registerType(workgrouptool,PROJECTNAME)



